@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import mock
+import netaddr
 import six
 import testtools
 
@@ -20,7 +22,7 @@ from neutron_lib.api import converters
 from neutron_lib import constants
 from neutron_lib import exceptions as n_exc
 from neutron_lib.tests import _base as base
-from neutron_lib.tests import _tools as tools
+from neutron_lib.tests import tools
 
 
 class TestConvertToBoolean(base.BaseTestCase):
@@ -200,6 +202,12 @@ class TestConvertIPv6AddrCanonicalFormat(base.BaseTestCase):
     def test_convert_invalid_address(self):
         result = converters.convert_ip_to_canonical_format("on")
         self.assertEqual("on", result)
+        result = converters.convert_ip_to_canonical_format(
+            u'192.168.1.1/32')
+        self.assertEqual(u'192.168.1.1/32', result)
+        result = converters.convert_ip_to_canonical_format(
+            u'2001:db8:0:1:1:1:1:1/128')
+        self.assertEqual(u'2001:db8:0:1:1:1:1:1/128', result)
 
 
 class TestConvertIPv6CIDRCanonicalFormat(base.BaseTestCase):
@@ -298,3 +306,50 @@ class TestConvertToString(base.BaseTestCase):
     def test_data_is_dict(self):
         self.assertEqual("{'foo': 'bar'}",
                          converters.convert_to_string({'foo': 'bar'}))
+
+
+class TestConvertUppercasePrefix(base.BaseTestCase):
+
+    def test_prefix_not_present(self):
+        self.assertEqual('foobar',
+                         converters.convert_prefix_forced_case('foobar',
+                                                               'bar'))
+
+    def test_prefix_no_need_to_replace(self):
+        self.assertEqual('FOObar',
+                         converters.convert_prefix_forced_case('FOObar',
+                                                               'FOO'))
+
+    def test_ucfirst_prefix_converted_1(self):
+        self.assertEqual('Foobar',
+                         converters.convert_prefix_forced_case('foobar',
+                                                               'Foo'))
+
+    def test_lc_prefix_converted_2(self):
+        self.assertEqual('foobar',
+                         converters.convert_prefix_forced_case('fOobar',
+                                                               'foo'))
+
+    def test_mixed_prefix_converted_1(self):
+        self.assertEqual('fOoXbar',
+                         converters.convert_prefix_forced_case('Fooxbar',
+                                                               'fOoX'))
+
+    def test_shorter_string(self):
+        self.assertEqual('fo',
+                         converters.convert_prefix_forced_case('fo',
+                                                               'foo'))
+
+
+class TestConvertPortMacAddress(base.BaseTestCase):
+
+    def test_mac_address_does_not_convert(self):
+        valid_mac = 'fa:16:3e:b6:78:1f'
+        self.assertEqual(valid_mac,
+                         converters.convert_to_mac_if_none(valid_mac))
+
+    @mock.patch('oslo_config.cfg.CONF')
+    def test_convert_none_to_mac_address(self, CONF):
+        CONF.base_mac = 'fa:16:3e:00:00:00'
+        self.assertTrue(
+            netaddr.valid_mac(converters.convert_to_mac_if_none(None)))
